package com.illescasDaniel.flutterShareItPlugin.share_it

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import androidx.core.content.FileProvider
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.io.File

class ShareItParameters(val map: Map<String, Any?>) {
  val type: String by map
  val title: String? by map
  val content: String by map
  val shareAsPlainText: Boolean by map
}

class ShareItPlugin(private val registrar: Registrar): MethodCallHandler {

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "share_it")
      channel.setMethodCallHandler(ShareItPlugin(registrar))
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "shareItAndroid" -> {
        val arguments = call.arguments
        if (arguments != null && arguments is Map<*,*>) {
          val parameters = ShareItParameters(map = mapOf(
            "type" to (call.argument<String>("type") ?: "*/*"),
            "title" to call.argument<String?>("title"),
            "content" to (call.argument<String>("content") ?: ""),
            "shareAsPlainText" to (call.argument<Boolean>("shareAsPlainText") ?: false)
          ))
          shareContent(parameters)
          result.success(true)
        } else {
          result.error("Incorrect arguments", null, null)
        }
      }
      else -> result.notImplemented()
    }
  }

  private fun shareContent(parameters: ShareItParameters) {
    val context = getActiveContext()
    val shareIntent = Intent().apply {
      action = Intent.ACTION_SEND
      if (parameters.type == "text/plain" && parameters.shareAsPlainText) {
        putExtra(Intent.EXTRA_TEXT, parameters.content)
      } else {
        val file = File(parameters.content)
        val authorities = context.packageName + ".shareit.fileprovider"
        val uri: Uri? =
          if (Build.VERSION.SDK_INT < Build.VERSION_CODES.N) Uri.fromFile(file)
          else FileProvider.getUriForFile(context, authorities, file)
        putExtra(Intent.EXTRA_STREAM, uri)
      }
      type = parameters.type
    }
    val title: CharSequence? = parameters.title
    context.startActivity(Intent.createChooser(shareIntent, title))
  }

  private fun getActiveContext(): Context {
    return if (registrar.activity() != null) registrar.activity() else registrar.context()
  }
}
