import Flutter
import UIKit

struct ShareItParameters: Decodable {
	enum ShareItType: String, Decodable {
		case text, image, uiImage, file, link
	}
	let content: String
	let type: ShareItType
}

public class SwiftShareItPlugin: NSObject, FlutterPlugin {

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "share_it", binaryMessenger: registrar.messenger())
    let instance = SwiftShareItPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

	if call.method == "shareItiOS" {
		guard let arguments = call.arguments else {
			result("Empty arguments")
			return
		}

		guard let shareItiOSData = try? JSONSerialization.data(withJSONObject: arguments, options: []) else {
			result("Invalid arguments")
			return
		}
		guard let parameters = try? JSONDecoder().decode(ShareItParameters.self, from: shareItiOSData) else {
			result("Error decoding arguments")
			return
		}

		activityController(parameters: parameters) {
			result(true)
		}
	}

  }

	// private

	private func activityItems(for parameters: ShareItParameters) -> [Any] {
		switch parameters.type {
		case .text:
			return [parameters.content]
		case .uiImage:
			guard FileManager.default.fileExists(atPath: parameters.content) else {
				return [parameters.content]
			}
			let url = URL(fileURLWithPath: parameters.content)
			guard let data = try? Data(contentsOf: url) else { return [] }
			guard let image = UIImage(data: data) else { return [] }
			return [image]
		case .link:
			let url = URL(string: parameters.content)
			return [url ?? parameters.content]
		case .image, .file:
			guard FileManager.default.fileExists(atPath: parameters.content) else {
				return [parameters.content]
			}
			let url = URL(fileURLWithPath: parameters.content)
			return [url]
		}
	}

	private func activityController(parameters: ShareItParameters, completionHandler: @escaping () -> Void) {
		let activityVC = UIActivityViewController(activityItems: activityItems(for: parameters), applicationActivities: nil)
		activityVC.popoverPresentationController?.sourceView = rootVC?.view
		if let view = rootVC?.view {
			activityVC.popoverPresentationController?.permittedArrowDirections = []
			activityVC.popoverPresentationController?.sourceRect = CGRect(x: view.bounds.midX, y: view.bounds.midY, width: 0, height: 0)
		}
		rootVC?.present(activityVC, animated: true, completion: completionHandler)
	}
	
	private var rootVC: UIViewController? {
		return UIApplication.shared.keyWindow?.rootViewController
	}
}
